# Input generation of vasp input files using config files and templates
"""
A module for generating vasp input and submission files using config files and templates.
"""
import os
import sys
import configparser as cp


def generate_input(incar_template):
    '''
    Generates the INCAR, and POTCAR files used to perform VASP calculations.

    Parameters
    ----------
    incar_template : str 

    Configuration files
    -------------------
    The incar_template file is a configuration file that contains the parameters for the INCAR file.
    The configuration file is a standard .ini file. The parameters are the same as those in the INCAR file.
    The parameters are read in as strings, and are converted to the appropriate type when the INCAR file is generated.
    The configuration files can be generated using the generate_config program.

    NOTE
    ----
    This program requires the use of a HOST configuration file.
    The HOST configuration file contains the information about the host machine.
    This includes:
        - The hostname
        - The architecture of the host machine and the subprocess it uses
        - The location of the POTPAW pseudopotential files on the host machine
    '''

    config = cp.ConfigParser()
    # create a variable for the vasp_templates directory inside the home directory and join it to the host.ini file 
    host_file = os.path.join(os.path.expanduser('~'),'vasp_templates','host.ini')
    config.read(host_file)
    hostname = config['host']['hostname']
    architecture = config['host']['architecture']
    potpaw_location = config['host']['potpaw']
    module_location = config['host']['module_location']
    config = cp.ConfigParser()
    input_file = os.path.join(os.path.expanduser('~'),'vasp_templates',incar_template)
    config.read(input_file)
    with open('INCAR', 'w') as f:
        for section in config.sections():
            for key in config[section]:
                key = key.upper()
                f.write(f'''{key} = {config[section][key]}
''')
    with open('POTCAR','a') as f:
        f.truncate(0)
        with open('POSCAR','r') as g:
            poscar = []
            for lines in g:
                stripped_lines = lines.strip()
                poscar.append(stripped_lines)
            atoms = poscar[5].split()
            potpaw = ['H', 'He', 'Li_sv', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne', 'Na_pv', 'Mg', 'Al', 'Si', 'P', 'S', 'Cl', 'Ar', 'K_sv', 'Ca_sv', 'Sc_sv', 'Ti_sv', 'V_sv', 'Cr_pv', 'Mn_pv', 'Fe', 'Co', 'Ni', 'Cu', 'Zn', 'Ga_d', 'Ge_d', 'As', 'Se', 'Br', 'Kr', 'Rb_sv', 'Sr_sv', 'Y_sv', 'Zr_sv', 'Nb_sv', 'Mo_sv', 'Tc_pv', 'Ru_pv', 'Rh_pv', 'Pd', 'Ag', 'Cd', 'In_d', 'Sn_d', 'Sb', 'Te', 'I', 'Xe', 'Cs_sv', 'Ba_sv', 'La', 'Ce_3', 'Nd_3', 'Pm_3', 'Sm_3', 'Eu_2', 'Gd_3', 'Tb_3', 'Dy_3', 'Ho_3', 'Er_3', 'Tm_3', 'Yb_2', 'Lu_3', 'Hf_pv', 'Ta_pv', 'W_sv', 'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl_d', 'Pb_d', 'Bi_d', 'Po_d', 'At', 'Rn', 'Fr_sv', 'Ra_sv', 'Ac', 'Th', 'Pa', 'U', 'Np', 'Pu', 'Am', 'Cm']
            potpaw_f = []
            for i in range(len(atoms)):
                for j in potpaw:
                    k = j.split('_')
                    if atoms[i] == k[0]:
                            potpaw_f.append(j)
            cwd = os.getcwd()
            os.system('cd ')
            os.system(f'module load {module_location}')
            os.chdir(potpaw_location)
            for i in potpaw_f:
                os.chdir(f'{i}')
                with open('POTCAR','r') as d:
                    for line in d:
                        f.write(line)
                os.chdir('../')
            os.chdir(cwd)

def generate_job(job_template, title):
    '''
    Generates the job submission script for the VASP calculation.

    Parameters
    ----------
    job_template : str
        The name of the job template file.
    title : str 

    Configuration files
    -------------------
    The job_template file is a configuration file that contains the parameters for the job submission script.
    The configuration file is a standard .ini file. The parameters are the same as those in the job submission script.
    The parameters are used to generate the job submission script based upon the subprocess 

    NOTE
    ----
    This program requires the use of a HOST configuration file.
    The HOST configuration file contains the information about the host machine.
    This includes:
        - The hostname
        - The architecture of the host machine and the subprocess it uses
        - The location of the POTPAW pseudopotential files on the host machine
    '''

    config = cp.ConfigParser()
    host_file = os.path.join(os.path.expanduser('~'),'vasp_templates','host.ini')
    config.read(host_file)
    hostname = config['host']['hostname']
    architecture = config['host']['architecture']
    potpaw_location = config['host']['potpaw']
    config = cp.ConfigParser()
    job_file = os.path.join(os.path.expanduser('~'),'vasp_templates',job_template)
    config.read(job_file)
    nodes = config['job']['nodes']
    cores = config['job']['cores']
    module_location = config['job']['module_location']
    vasp_type = config['job']['vasp_type']
    if architecture == 'slurm':
        with open('submit.sh','w') as f:
            f.truncate(0)
            f.write(f'''#!/bin/bash
#SBATCH -p {nodes}       
#SBATCH -n {cores}
#SBATCH --job-name="{title}"

echo "Starting run at: `date`"

module load {module_location}
module load anaconda3/2020.07

# Call VASP directly.

mpirun -np {cores} {vasp_type}

echo "Job finished with exit code $? at: `date`"
''')
      
        with open('Opt.sh','w') as f:
            f.write(f'''#!/bin/bash --login
#SBATCH -p {nodes}
#SBATCH -n {cores}
#SBATCH --job-name="{title}"

# Check to see if the required files to run vasp are in the cwd if not exit job. 
if test -e 'INCAR' && test -e 'POSCAR' && test -e 'POTCAR'; then
   echo "Submission files located"
else
   echo "Submission files missing exit with code 1 at: `date`"
   exit 1;
fi

module load {module_location}
module load anaconda3/2020.07

run_num=0
running=True
while running=True
do
    
    echo "Started running at: `date`"
    
    mpirun -np {cores} {vasp_type} >> vasp.out

    let run_num++

    conv=$(grep "reached required accuracy" vasp.out | wc -l)

    # check to see if the calculation has converged
''')
        with open('Opt.sh','a') as f:
            f.write('''
    if [[ "$conv" == 1 ]]; then
        E=$(awk '/F=/ {print}' OSZICAR)
        echo 'RUN NUMBER: ' $run_num
        echo $E
        echo "Job finished with exit code 0 at: `date`"
        mkdir  Opt-$run_num
        cp {CHG,CHGCAR,CONTCAR,DOSCAR,EIGENVAL,IBZKPT,INCAR,KPOINTS,OSZICAR,OUTCAR,PCDAT,POSCAR,POTCAR,REPORT,vasp.out,WAVECAR,XDATCAR} Opt-$run_num
        cp CONTCAR POSCAR.opt
        rm CHG CHGCAR CONTCAR DOSCAR EIGENVAL IBZKPT OSZICAR OUTCAR PCDAT REPORT WAVECAR XDATCAR vasprun.xml
        break
     elif [[ "$run_num" == 15 ]]; then
        E=$(awk '/F=/ {print}' OSZICAR)
        echo 'RUN NUMBER: ' $run_num
        echo $E
        echo "Running iteration limit reached: 15, Job finished with exit code 0 at: `date`"
        cp {CHG,CHGCAR,CONTCAR,DOSCAR,EIGENVAL,IBZKPT,INCAR,KPOINTS,OSZICAR,OUTCAR,PCDAT,POSCAR,POTCAR,REPORT,vasp.out,WAVECAR,XDATCAR} Opt-$run_num
        cp CONTCAR POSCAR
        rm CHG CHGCAR CONTCAR DOSCAR EIGENVAL IBZKPT OSZICAR OUTCAR PCDAT REPORT WAVECAR XDATCAR vasprun.xml
        break
    else
        E=$(awk '/F=/ {print}' OSZICAR)
        echo 'RUN NUMBER: ' $run_num
        echo $E
        mkdir  Opt-$run_num
        cp {CHG,CHGCAR,CONTCAR,DOSCAR,EIGENVAL,IBZKPT,INCAR,KPOINTS,OSZICAR,OUTCAR,PCDAT,POSCAR,POTCAR,REPORT,vasp.out,WAVECAR,XDATCAR} Opt-$run_num
        cp CONTCAR POSCAR
        rm CHG CHGCAR CONTCAR DOSCAR EIGENVAL IBZKPT OSZICAR OUTCAR PCDAT REPORT vasp.out WAVECAR XDATCAR vasprun.xml
        echo "Number of iterations reached NSW, job finished at: `date`"
        echo " "
        echo " "
    fi
done
''')
    elif architecture == 'sge':
         if nodes == 'multicore':
             with open('submit.sh','w') as f:
                 f.truncate(0)
                 f.write(f'''#!/bin/bash --login
#$ -cwd             # Job will run from the current directory
#$ -pe smp.pe {cores}
#$ -N '{title}'

echo "Starting run at: `date`"

module load {module_location}

mpirun -n $NSLOTS {vasp_type}

echo "Job finished with exit code $? at: `date`"
''')
         if nodes == 'multinode':
             with open('submit.sh','w') as f:
                 f.truncate(0)
                 f.write(f'''#!/bin/bash --login
#$ -cwd                       # Job will run from the current directory
#$ -pe mpi-24-ib.pe {cores}
#$ -N '{title}'

module load {module_location}

mpirun -n $NSLOTS {vasp_type}

echo "Job finished with exit code $? at: `date`"               
''')
         with open('Opt.sh','w') as f:
             f.write(f'''#!/bin/bash --login
#$ -cwd             
#$ -pe smp.pe {cores}
#$ -N '{title}'

# Check to see if the required files to run vasp are in the cwd if not exit job. 
if test -e 'INCAR' && test -e 'POSCAR' && test -e 'POTCAR'; then
   echo "Submission files located"
else
   echo "Submission files missing exit with code 1 at: `date`"
   exit 1;
fi

module load {module_location}
module load anaconda3/2020.07

run_num=0
running=True
while running=True
do
    
    echo "Started running at: `date`"
    
    mpirun -np {cores} {vasp_type} >> vasp.out

    let run_num++

    conv=$(grep "reached required accuracy" vasp.out | wc -l)

    # check to see if the calculation has converged
''')
         with open('Opt.sh','a') as f:
             f.write('''
    if [[ "$conv" == 1 ]]; then
        E=$(awk '/F=/ {print}' OSZICAR)
        echo 'RUN NUMBER: ' $run_num
        echo $E
        echo "Job finished with exit code 0 at: `date`"
        mkdir  Opt-$run_num
        cp {CHG,CHGCAR,CONTCAR,DOSCAR,EIGENVAL,IBZKPT,INCAR,KPOINTS,OSZICAR,OUTCAR,PCDAT,POSCAR,POTCAR,REPORT,vasp.out,WAVECAR,XDATCAR} Opt-$run_num
        cp CONTCAR POSCAR.opt
        rm CHG CHGCAR CONTCAR DOSCAR EIGENVAL IBZKPT OSZICAR OUTCAR PCDAT REPORT WAVECAR XDATCAR vasprun.xml
        break
     elif [[ "$run_num" == 15 ]]; then
        E=$(awk '/F=/ {print}' OSZICAR)
        echo 'RUN NUMBER: ' $run_num
        echo $E
        echo "Running iteration limit reached: 15, Job finished with exit code 0 at: `date`"
        cp {CHG,CHGCAR,CONTCAR,DOSCAR,EIGENVAL,IBZKPT,INCAR,KPOINTS,OSZICAR,OUTCAR,PCDAT,POSCAR,POTCAR,REPORT,vasp.out,WAVECAR,XDATCAR} Opt-$run_num
        cp CONTCAR POSCAR
        rm CHG CHGCAR CONTCAR DOSCAR EIGENVAL IBZKPT OSZICAR OUTCAR PCDAT REPORT WAVECAR XDATCAR vasprun.xml
        break
    else
        E=$(awk '/F=/ {print}' OSZICAR)
        echo 'RUN NUMBER: ' $run_num
        echo $E
        mkdir  Opt-$run_num
        cp {CHG,CHGCAR,CONTCAR,DOSCAR,EIGENVAL,IBZKPT,INCAR,KPOINTS,OSZICAR,OUTCAR,PCDAT,POSCAR,POTCAR,REPORT,vasp.out,WAVECAR,XDATCAR} Opt-$run_num
        cp CONTCAR POSCAR
        rm CHG CHGCAR CONTCAR DOSCAR EIGENVAL IBZKPT OSZICAR OUTCAR PCDAT REPORT vasp.out WAVECAR XDATCAR vasprun.xml
        echo "Number of iterations reached NSW, job finished at: `date`"
        echo " "
        echos " "
    fi
done
''')
    elif architecture == 'local':
        with open('submit.sh','w') as f:
            f.truncate(0)
            f.write(f'''nohup mpirun -n {cores} {vasp_type} > output01.txt &''')

    else:
        print('Architecture not supported')
    



# The command line interface for vasp_suite
'''
This module contains the command line interface for vasp_suite.
'''

# Imports
import argparse
from textwrap import dedent

# Import the package moddules
from . import input
from . import plotter
from . import file_transform
from . import config

# Wrapper functions for argparse
def generate_input_func(args):
    '''
    Wrapper function for the command line interface to call generate_input
    Parameters
    ----------
    args : argparser object
        command line arguments 
    Returns 
    -------
    None
    '''
    input.generate_input(
            incar_template=args.incar_template)

def generate_job_func(args):
    '''
    Wrapper function for the command line interface to call genrate_job 

    Parameters
    ----------
    args : argparser object
        command line arguments 

    Returns 
    -------
    None
    '''
    input.generate_job(
            job_template=args.job_template,
            title=args.title
            )

def convert_cif_func(args):
    '''
    Wrapper function for the command line interface to call convert_cif

    Parameters
    ----------
    args : argparser object
        command line arguments 

    Returns 
    -------
    None
    '''
    file_transform.convert_cif(
            cif_file=args.cif_file)

def convert_xyz_func(args):
    '''
    Wrapper function for the command line interface to call convert_xyz
    
    Parameters
    ----------
    No arguments required

    Returns 
    -------
    None
    '''
    file_transform.convert_xyz()

def plot_dos_func(args):
    '''
    Wrapper function for the command line interface to call plot_dos

    Parameters
    ----------
    args : argparser object
        command line arguments 

    Returns 
    -------
    None
    '''
    plotter.plot_dos(
            dos_type=args.dos_type,
            orbitals=args.orbitals)

def generate_config_func(args):
    '''
    Wrapper function for the command line interface to call generate_config_script

    Parameters
    ----------
    args : argparser object
        command line arguments 

    Returns 
    -------
    None
    '''
    config.generate_config_script(
           config =  args.config_type)

def supercell_func(args):
    '''
    Wrapper function for the command line interface to call  supercell_expansion

    Parameters
    ----------
    args : argparser object
        command line arguments 

    Returns 
    -------
    None
    '''
    file_transform.supercell_expansion(
            a = args.a,
            b = args.b,
            c = args.c)

# Parser
def read_args(arg_list=None):
    """Reads the command line arguments and returns them as a dictionary"""
    parser = argparse.ArgumentParser(
        prog='vasp_suite',
        description= dedent(
            '''
            A package for dealing with VASP calculations.
            The package can generate input files, it can perform file transformations including:
                 - .cif
                 - .xyz
                 - POSCAR

             The package contains analysis tools for plotting density of states calculated in vasp.
                            
             Available programs:
                 vasp_suite generate_input ...
                 vasp_suite generate_job ...
                 vasp_suite convert_cif ...
                 vasp_suite convert_xyz ...
                 vasp_suite plot_dos ...
                 vasp_suite generate_config ...
                 vasp_suite supercell ...
                 '''
                 ),
        epilog = dedent(
            '''To display options for a specific program, use vasp_suite <program> -h'''
            ),

        formatter_class=argparse.RawDescriptionHelpFormatter
        )


    subparsers = parser.add_subparsers(dest='prog')

    gen_inp = subparsers.add_parser(
            'generate_input',
            formatter_class=argparse.RawDescriptionHelpFormatter
            )
    gen_inp.set_defaults(func=generate_input_func)

    gen_inp.add_argument(
            'incar_template',
            help=dedent("The template (.ini) file the INCAR file will be generated from"),
            type=str,
            default='input.ini'
            )

    gen_job = subparsers.add_parser(
            'generate_job',
            formatter_class=argparse.RawDescriptionHelpFormatter
            )
    gen_job.set_defaults(func=generate_job_func)

    gen_job.add_argument(
            'job_template',
            help=dedent('The template (.ini) file the job script will be generated from.'),
            type=str,
            default='job.ini'
            )
    gen_job.add_argument(
            'title',
            help=dedent('The title of the job. This will be used to name the output files.'),
            type=str,
            default='job'
            )

    conv_cif = subparsers.add_parser(
            'convert_cif',
            formatter_class=argparse.RawDescriptionHelpFormatter
            )
    conv_cif.set_defaults(func=convert_cif_func)

    conv_cif.add_argument(
            'cif_file',
            help="The .cif file to be converted to POSCAR",
            type=str,
            default='POSCAR'
            )

    conv_xyz = subparsers.add_parser(
            'convert_xyz',
            formatter_class=argparse.RawDescriptionHelpFormatter
            )

    conv_xyz.set_defaults(func=convert_xyz_func)

    dos = subparsers.add_parser(
            'plot_dos',
            formatter_class=argparse.RawDescriptionHelpFormatter
            )
    dos.set_defaults(func=plot_dos_func)
    
    dos.add_argument(
            'dos_type',
            help="The type of DOS to be plotted (total, orbital)",
            type=str,
            default='total'
            )

    dos.add_argument(
            '--orbitals',
            help="The orbitals to be plotted (d, f)",
            type=str,
            default='d'
            )

    conf = subparsers.add_parser(
            'generate_config',
            formatter_class=argparse.RawDescriptionHelpFormatter
            )

    conf.set_defaults(func=generate_config_func)

    conf.add_argument(
            'config_type',
            help=dedent("The type of config file to be generated (job, input, host)"),
            type = str
            )

    super = subparsers.add_parser(
            'supercell',
            formatter_class=argparse.RawDescriptionHelpFormatter
            )

    super.set_defaults(func=supercell_func)

    super.add_argument(
            'a',
            help=dedent("The number of unit cells in the a direction"),
            type = int
            )

    super.add_argument(
            'b',
            help=dedent("The number of unit cells in the b direction"),
            type = int
            )

    super.add_argument(
            'c',
            help=dedent("The number of unit cells in the c direction"),
            type = int
            )


    # Read sub-parser
    parser.set_defaults(func=lambda args: parser.print_help())
    args = parser.parse_known_args(arg_list)

    #select program
    if args in ['generate_input', 'generate_job', 'convert_cif', 'convert_xyz', 'plot_dos', 'generate_config', 'supercell']:
        args.func(args)
    else:
        args = parser.parse_args(arg_list)
        args.func(args)

def main():
    read_args()
            





# The following module allows for the transformation of files for use and analysis in vasp and other codes such as molcas
# The module will include the transfor from .cif file to poscar file and then poscar to .xyz file 
"""
A module for the transformation of file type for use in vasp and other codes such as molcas
"""
# Imports
import numpy as np
import os
from gemmi import cif

# Functions
def convert_cif(cif_file):
    """
    Converts a .cif file to a POSCAR file for use in vasp.
    KNOWN BUGS
    ----------
    - The program doesnt work for .cif files containig "(number)" in the positions of the atoms and the lattice vectors
    - View the .cif file in a text editor to see if this is the case

    Parameters
    ----------
    cif_file : str 

    Returns
    -------
    POSCAR 
    """

    cif_doc =  cif.read_file(cif_file)
    cif_block = cif_doc.sole_block()
    name = cif_file.strip(".cif")
    # Get the lattice vectors
    a = cif_block.find_value("_cell_length_a")
    b = cif_block.find_value("_cell_length_b")
    c = cif_block.find_value("_cell_length_c")
    a, b, c = float(a), float(b), float(c)
    alpha = cif_block.find_value("_cell_angle_alpha")
    beta = cif_block.find_value("_cell_angle_beta")
    gamma = cif_block.find_value("_cell_angle_gamma")
    alpha, beta, gamma = float(alpha), float(beta), float(gamma)
    alpha, beta, gamma = np.radians(alpha), np.radians(beta), np.radians(gamma)
    # Get the atom positions
    xpos, ypos, zpos = list(cif_block.find_loop("_atom_site_fract_x")), list(cif_block.find_loop("_atom_site_fract_y")), list(cif_block.find_loop("_atom_site_fract_z"))
    # Get the atom labels
    atom_labels = list(cif_block.find_loop("_atom_site_label"))
    # Get the atom types
    atom_types = list(cif_block.find_loop("_atom_site_type_symbol"))
    
    # Transform the lattice vectors and angles into a matix
    a_vector = np.array([a, 0, 0])
    b_vector = np.array([b*np.cos(gamma), b*np.sin(gamma), 0])
    c_vector = np.array([np.cos(beta), c*((np.cos(alpha) - np.cos(beta)*np.cos(gamma))/np.sin(gamma)), c*np.sqrt(1 - np.cos(beta)**2 - ((np.cos(alpha) - np.cos(beta)*np.cos(gamma))/np.sin(gamma))**2)])

    # Create the atom position matirx
    position_matrix = [] 
    for i in range(len(xpos)):
        position = np.array([float(xpos[i]), float(ypos[i]), float(zpos[i])])
        position_matrix.append(position)
    position_matrix = np.array(position_matrix)
    
    # Create a list of tuples containing the atom and the number of that atom (atom, number)
    atom_list = []
    for i in atom_types:
        if i not in atom_list:
            atom_list.append(i)
    atom_number_list = []
    for i in atom_list:
        atom_number_list.append(atom_types.count(i))
    atom_number_list = tuple(atom_number_list)
    atom_list = tuple(atom_list)
    atom_list = list(zip(atom_list, atom_number_list))

    atom_list_string = ""
    for i in atom_list:
        atom_list_string += i[0] + " "
    atom_number_list_string = ""
    for i in atom_list:
        atom_number_list_string += str(i[1]) + " "

    # Generate the POSCAR file
    with open('POSCAR', 'w') as f:
        f.write(f'''{name}
1.0
        {a_vector[0]} {a_vector[1]} {a_vector[2]}
        {b_vector[0]} {b_vector[1]} {b_vector[2]}
        {c_vector[0]} {c_vector[1]} {c_vector[2]}
{atom_list_string}
  {atom_number_list_string}
Direct
''')
        for i in position_matrix:
            f.write(f"    {i[0]}     {i[1]}      {i[2]}\n")


def convert_xyz():
    """
    Converts a POSCAR file to a .xyz file for use in molcas

    Parameters
    ----------
    None

    Returns
    -------
    .xyz file
    """
    with open('POSCAR','r') as f:
            poscar = []
            for lines in f:
                stripped_lines = lines.strip()
                split_lines = stripped_lines.split()
                poscar.append(split_lines)
    lattice_matrix = np.float_(poscar[2:5])
    frac_coord = np.float_(poscar[8:])
    name = ''.join(poscar[0])
    cart_coord = []
    for i in range(len(frac_coord)):
        cart_coord.append(np.matmul(frac_coord[i],lattice_matrix))
    element = poscar[5]
    element_num = poscar[6]
    element_num = [int(x) for x in element_num]
    cumsum_element = list(np.cumsum(element_num))
    element_list =[]
    for i in range(len(element)):
        element_list.append(f'{element[i]} '*int(element_num[i]))
    for i in range(len(element_list)):
        element_list[i] = ' '.join(element_list[i].split())
    element_list = ' '.join(element_list)
    element_list = element_list.split()

    with open('POSCAR.xyz','w') as f:
        f.write(f'''{len(cart_coord)}
{name}
''')
        for i in range(len(cart_coord)):
            f.write(f'''{element_list[i]}    {float(cart_coord[i][0])}    {float(cart_coord[i][1])}    {float(cart_coord[i][2])}   
''')  

def get_poscar():
    poscar = []
    with open('POSCAR', 'r') as f:
        for lines in f:
            strippedlines = lines.strip()
            splitlines = strippedlines.split()
            poscar.append(splitlines)
    return poscar

def atom_number(poscar):
    atom_number = np.int_(poscar[6])
    return atom_number

def atom_list(poscar):
    atom_list = poscar[5]
    atom_number = list(np.int_(poscar[6]))
    atoms = []
    for atom, number in zip(atom_list, atom_number):
        for i in range(number):
            atoms.append(atom)
    return atoms

def construct_matrix(poscar):
    matrix = np.float_(poscar[8:])
    return matrix

def expansion_list(a,b,c):
    a_list = list(range(a+1))
    del a_list[0]
    a_list.append(0)
    b_list = list(range(b+1))
    del b_list[0]
    b_list.append(0)
    c_list = list(range(c+1))
    del c_list[0]
    c_list.append(0)
    return a_list,b_list, c_list       

def expansion(matrix, a_list, b_list, c_list, atoms):
    mat = []
    for item, atom in zip(matrix, atoms):
        item = np.append(item,[atom])
        mat.append(item)
    mat = np.array(mat)
    a_matrix = []
    b_matrix = []
    c_matrix = []
    for i in a_list[:-1]:
        for j in mat:
                a_matrix.append([((float(i)*1)+float(j[0])), float(j[1]),float(j[2]), j[3]])
    a_matrix = np.array(a_matrix)
    a_matrix = np.unique(a_matrix,axis=0)
    a_matrix = list(a_matrix)
    for i in b_list[:-1]:
        for j in a_matrix:
            b_matrix.append([float(j[0]), ((float(i)*1)+float(j[1])), float(j[2]),j[3]])
    b_matrix = np.array(b_matrix)
    b_matrix = np.unique(b_matrix,axis=0)
    b_matrix = list(b_matrix)
    for i in c_list[:-1]:
        for j in b_matrix:
            c_matrix.append([float(j[0]), float(j[1]), ((float(i)*1)+float(j[2])),j[3]])
    matrix = np.array(c_matrix)
    matrix = np.unique(matrix, axis=0)
    matrix = matrix[matrix[:,3].argsort()]
    return matrix

def fractional_matrix(matrix, a, b,c):
    for i in range(len(matrix)):
        matrix[i][0] = float(matrix[i][0])/a[-2]
        matrix[i][1] = float(matrix[i][1])/b[-2]
        matrix[i][2] = float(matrix[i][2])/c[-2]
    return matrix

def new_atoms(matrix):
    atoms = []
    for i in matrix:
        atoms.append(i[3])
    #append the number of each atom to a new list
    atom_number = []
    for i in atoms:
        atom_number.append(atoms.count(i))
    #remove the duplicate atoms
    atom_list_sc = []
    for i in atoms:
        if i not in atom_list_sc:
            atom_list_sc.append(i)
    #remove duplicate atom numbers
    atom_number_sc = []
    # calcualte the number of times that each atom in atom_liat_sc appears in atoms and append it to atom_number_sc
    for i in atom_list_sc:
        atom_number_sc.append(atoms.count(i))
    return atom_list_sc, atom_number_sc

def multiplied_lattice_matrix(a, b, c, poscar):
    lattice_matrix = poscar[2:5]
    lattice_matrix = np.float_(lattice_matrix)
    for i in range(len(lattice_matrix[0])):
        lattice_matrix[0][i] = lattice_matrix[0][i]*a[-2]
    for i in range(len(lattice_matrix[1])):
        lattice_matrix[1][i] = lattice_matrix[1][i]*b[-2]
    for i in range(len(lattice_matrix[2])):
        lattice_matrix[2][i] = lattice_matrix[2][i]*c[-2]
    return lattice_matrix

def write_poscar(matrix, lattice_matrix, atom_list, atom_number,name):
    with open('POSCAR', 'w') as f:
        f.write(f'''{name}
1.0
                {lattice_matrix[0][0]} {lattice_matrix[0][1]} {lattice_matrix[0][2]}
                {lattice_matrix[1][0]} {lattice_matrix[1][1]} {lattice_matrix[1][2]}
                {lattice_matrix[2][0]} {lattice_matrix[2][1]} {lattice_matrix[2][2]}
''')
        atom_list = ' '.join(atom_list)
        f.write(f'''  {atom_list}
''')
        atom_number = ' '.join(map(str, atom_number))
        f.write(f'''  {atom_number}
''')
        f.write('''Dierct
''')
        for i in matrix:
            f.write(f''' {i[0]} {i[1]} {i[2]}
''')

def supercell_expansion(a,b,c):
    '''
    A program to perform supercell expansions on a POSCAR file.
    where a, b and c are the expansion factors in the a, b and c directons respectively.

    Parameters
    ----------
    a : int 
    b : int 
    c : int 

    Returns
    -------
    POSCAR 

    '''
    poscar = get_poscar()
    name = poscar[0]
    matrix = construct_matrix(poscar)
    a,b,c = expansion_list(a,b,c)
    supercell_lattice_matrix = multiplied_lattice_matrix(a,b,c, poscar)
    atoms = atom_list(poscar)
    new_matrix = expansion(matrix,a,b,c,atoms)
    new_matrix = fractional_matrix(new_matrix, a,b,c)
    atom_list_sc, atom_number_sc = new_atoms(new_matrix)
    write_poscar(new_matrix, supercell_lattice_matrix, atom_list_sc, atom_number_sc, name)


